unit uOOoDatasetRowPrinter;
{                                                                             |
    Part of package pOpenOfficeReport                                         |
                                                                              |
    Author: Jean SUZINEAU <Jean.Suzineau@wanadoo.fr>                          |
            partly as freelance: http://www.mars42.com                        |
        and partly as employee : http://www.batpro.com                        |
    Contact: gilles.doutre@batpro.com                                         |
                                                                              |
    Copyright (C) 2004  Jean SUZINEAU - MARS42                                |
    Copyright (C) 2004  Cabinet Gilles DOUTRE - BATPRO                        |
                                                                              |
    See pOpenOfficeReport.dpk for full copyright notice.                      |
|                                                                             }
interface

uses
    SysUtils, Classes, DB,Variants, Dialogs,ComObj,Forms,
    uUNO_ControlCharacter,
    uUNO_DeskTop,
    uUNO_BookMark,
    uUNO_FieldMaster_User,
    uUNO_TextField_User,
    udllOpenOfficeReport_Register;

type
 TOOoDatasetRowPrinter
 =
  class
  private
    function ComposeFileName( D: TDateTime; NomFichierModele: String): String;
    function old_BookMark_Execute( NomFichierModele: String; D: TDataset): Boolean;
  public
    procedure AssureModele( NomFichierModele: String; D: TDataset; Mettre_a_jour: Boolean= False);
    function Execute( NomFichierModele: String; D: TDataset; Enregistrer: Boolean): Variant;
    function Execute_for_each_row( NomFichierModele: String; D: TDataset): Boolean;
  end;

var
   OOoDatasetRowPrinter: TOOoDatasetRowPrinter= nil;

implementation

uses uOOoTextFieldsCreator;

procedure Detruire( F: Variant);
var
   Detruire: Boolean;
begin
     Detruire:= True;
     while Detruire
     do
       try
          F.Close( True);
          Detruire:= False;
       except
             on E: EOleException
             do
               begin
               //ShowMessage( 'OOoDatasetRowPrinter.Execute_for_each_row'#13#10+
               //             E.Message);
               end;
             end;
end;

{ TOOoDatasetRowPrinter }

function TOOoDatasetRowPrinter.ComposeFileName( D: TDateTime;
                                                NomFichierModele: String): String;
var
   ModelPath, ModelName, ModelExt: String;
   Anne_Mois_Jour: String;
   NomJour: String;
   Heure, Minute: String;
   L: Integer;
begin
     Anne_Mois_Jour:= FormatDateTime( 'yyyy_mm_dd', D);
     NomJour       := FormatDateTime( 'ddd'       , D);
     Heure         := FormatDateTime( 'hh'        , D);
     Minute        := FormatDateTime( 'nn'        , D);

     L:= Length( NomJour);
     if L > 0
     then
         if NomJour[L] = '.'
         then
             Delete( NomJour, L, 1);

     ModelPath:= ExtractFilePath( NomFichierModele);
     ModelName:= ExtractFileName( NomFichierModele);
     ModelName:= ChangeFileExt( ModelName, '');
     ModelExt:= UpperCase( ExtractFileExt( NomFichierModele));

     Result
     :=
       ModelPath+
       Anne_Mois_Jour+'_'+NomJour+'_'+Heure+'h'+Minute+'_'+ModelName;

     if ModelExt = '.OTT'
     then
         Result:= Result+'.ODT'
     else
         ShowMessage( ModelExt+
                      ': Type de modle non gr par pOpenOfficeReport::TOOoDatasetRowPrinter.ComposeFileName');

end;

function TOOoDatasetRowPrinter.old_BookMark_Execute( NomFichierModele: String; D: TDataset): Boolean;
var
   NbColonnes: Integer;
   I: Integer;
   Field: TField;
   sFieldLabel,
   sFieldValue: String;

   URL: String;

   F,
   FieldBookMark,
   FieldBookMarkAnchor: Variant;
begin
     URL:= UNO_DeskTop.URL_from_WindowsFileName( ComposeFileName( Now, NomFichierModele));
     NbColonnes:= D. FieldCount;

     F:= UNO_DeskTop.OpenFile( NomFichierModele, False);

     for I:= 0 to NbColonnes-1
     do
       begin
       Field:= D.Fields[I];
       sFieldLabel:= Field.DisplayLabel;
       sFieldValue:= Field.DisplayText;

       FieldBookMark:= F.getBookmarks.getByName( sFieldLabel);
       FieldBookMarkAnchor:= FieldBookMark.getAnchor;
       FieldBookMarkAnchor.setString( sFieldValue);
       end;

     F.storeAsURL( URL, VarArrayOf([]));
     Result:= True;
end;

procedure TOOoDatasetRowPrinter.AssureModele( NomFichierModele: String;
                                              D: TDataset;
                                              Mettre_a_jour: Boolean= False);
var
   NouveauModele: Boolean;
   URLModele: String;

   F: Variant;
begin
     NouveauModele:= not FileExists( NomFichierModele);
     if not (NouveauModele or Mettre_a_jour)
     then
         exit;

     URLModele:= UNO_DeskTop.URL_from_WindowsFileName( NomFichierModele);

     if NouveauModele
     then
         F:= UNO_DeskTop.CreateWriter
     else
         F:= UNO_DeskTop.OpenFile( NomFichierModele, False, False);

     OOoTextFieldsCreator.Execute_Modele ( F, D, NouveauModele);

     F.storeAsURL( URLModele, VarArrayOf([]));
     F.Close( True);
     //F.Dispose;
end;

function TOOoDatasetRowPrinter.Execute( NomFichierModele: String; D: TDataset; Enregistrer: Boolean): Variant;
var
   URL: String;
   F: Variant;
begin
     AssureModele( NomFichierModele, D, True);

     F:= UNO_DeskTop.OpenFile( NomFichierModele, False);

     OOoTextFieldsCreator.Execute( F, D);

     if Enregistrer
     then
         begin
         URL
         :=
           UNO_DeskTop.URL_from_WindowsFileName( ComposeFileName(Now,NomFichierModele));
         F.storeAsURL( URL, VarArrayOf([]));
         end;
     Result:= F;
end;

function TOOoDatasetRowPrinter.Execute_for_each_row( NomFichierModele: String; D: TDataset): Boolean;
var
   F: Variant;
   PrintOptions: Variant;
   PrintCount: Integer;
   Listener: Variant;
   LastState: Integer;
begin
     PrintOptions:= VarArrayCreate([0, -1], varVariant);

     AssureModele( NomFichierModele, D, True);

     F:= UNO_DeskTop.OpenFile( NomFichierModele, False);
     Listener:= CreateOleObject( 'dllOpenOfficeAutomation.UNOPrintJobListener');
     F.addPrintJobListener( Listener);

     PrintCount:= 0;
     D.First;
     while not D.Eof
     do
       begin
       OOoTextFieldsCreator.Execute( F, D);
       F.Print( PrintOptions);
       LastState:= 0;
       while LastState < 1
       do
         begin
         Application.ProcessMessages;
         LastState:= Listener.LastState;
         end;
       Inc( PrintCount);
       D.Next;
       end;

     F.Close( True);
     ShowMessage( Format('%d documents imprims', [PrintCount]));
     Result:= True;
end;

initialization
              OOoDatasetRowPrinter:= TOOoDatasetRowPrinter.Create;
finalization
              FreeAndNil( OOoDatasetRowPrinter);
end.
