unit uOOoTextFieldsCreator;
{                                                                             |
    Part of package pOpenOfficeReport                                         |
                                                                              |
    Author: Jean SUZINEAU <Jean.Suzineau@wanadoo.fr>                          |
            partly as freelance: http://www.mars42.com                        |
        and partly as employee : http://www.batpro.com                        |
    Contact: gilles.doutre@batpro.com                                         |
                                                                              |
    Copyright (C) 2004  Jean SUZINEAU - MARS42                                |
    Copyright (C) 2004  Cabinet Gilles DOUTRE - BATPRO                        |
                                                                              |
    See pOpenOfficeReport.dpk for full copyright notice.                      |
|                                                                             }

interface

uses
    SysUtils, Classes, DB,
    uUNO_ControlCharacter,
    uUNO_DeskTop,
    uUNO_FieldMaster_User,
    uUNO_TextField_User;

type
 TOOoTextFieldsCreator
 =
  class
  //Gnral
  private
    Document,
    Text,
    Cursor,
    TFM,//TextFieldMasters
    TextFields: Variant;
    procedure Init( _Document: Variant);
    function Traite_Field( FieldName, FieldContent: String;
                           CreeTextFields: Boolean): Boolean;
  //cration de champs  partir de tous les champs d'un dataset
  private
    function Execute_interne( _Document: Variant;
                              D: TDataset;
                              UtiliseValeurs, CreeTextFields: Boolean): Boolean;
                              overload;
  public
    //juste pour les tests
    function OOoFieldName_from_DisplayLabel( DisplayLabel: String): String;

    function Execute_Modele( _Document: Variant;
                             D: TDataset; Nouveau_Modele: Boolean): Boolean;
                             overload;
    function Execute       ( _Document: Variant;
                             D: TDataset): Boolean;
                             overload;
  //cration d'un seul champ
  private
    function Execute_interne( _Document: Variant;
                              FieldName, FieldContent: String;
                              CreeTextFields: Boolean): Boolean;
                              overload;
  public
    function Execute_Modele( _Document: Variant;
                             FieldName: String;
                             Nouveau_Modele: Boolean): Boolean;
                             overload;
    function Execute       ( _Document: Variant;
                             FieldName, FieldContent: String): Boolean;
                             overload;
  //Contrle d'existence d'un champ
  public
    function Existe( _Document: Variant; FieldName: String): Boolean;
  end;

var
   OOoTextFieldsCreator: TOOoTextFieldsCreator= nil;

implementation

{ TOOoTextFieldsCreator }

function TOOoTextFieldsCreator.OOoFieldName_from_DisplayLabel( DisplayLabel: String): String;
var
   I: Integer;
begin
     Result:= DisplayLabel;
     for I:= 1 to Length( Result)
     do
       if Result[I] = ' ' then Result[I]:= '_';
end;

procedure TOOoTextFieldsCreator.Init( _Document: Variant);
begin
     Document:= _Document;
     Text:= Document.Text;
     Cursor:= Text.createTextCursor;
     TFM       := Document.TextFieldMasters;
     TextFields:= Document.TextFields;
end;


function TOOoTextFieldsCreator.Traite_Field( FieldName,
                                             FieldContent: String;
                                             CreeTextFields: Boolean
                                             ): Boolean;
var
   FieldPathName: String;
   fm: TUNO_FieldMaster_User;
   tf: TUNO_TextField_User;
begin
     FieldPathName:= 'com.sun.star.text.FieldMaster.User.'+FieldName;

     if TFM.hasByName( FieldPathName)
     then
         fm:=TUNO_FieldMaster_User.Create_From(Document,TFM.getByName(FieldPathName))
     else
         fm:=TUNO_FieldMaster_User.Create     (Document,FieldName);
     try
        fm.Content:= FieldContent;

        if CreeTextFields
        then
            begin
            tf:= TUNO_TextField_User.Create( Document);
            try
               tf.attachTextFieldMaster( fm);

               Cursor.gotoEnd( False);
               Text.insertControlCharacter( Cursor,
                                            UNO_ControlCharacter_PARAGRAPH_BREAK,
                                            False);
               Cursor.gotoEnd( False);
               Text.getEnd.SetString( FieldName+': ');

               Cursor.gotoEnd( False);
               Text.insertTextContent( Text.End, tf.ole, False);
            finally
                   FreeAndNil( tf);
                   end;
            end;
     finally
            FreeAndNil( fm);
            end;
     Result:= True;
end;

function TOOoTextFieldsCreator.Execute_interne( _Document: Variant;
                                                D: TDataset;
                                                UtiliseValeurs,
                                                CreeTextFields: Boolean): Boolean;
var
   NbColonnes: Integer;
   I: Integer;
   Field: TField;
   sOOoFieldName    ,
   sOOoFieldContent : String;
begin
     NbColonnes:= D.FieldCount;

     Init( _Document);

     for I:= 0 to NbColonnes-1
     do
       begin
       Field:= D.Fields[I];
       sOOoFieldName   := OOoFieldName_from_DisplayLabel( Field.DisplayLabel);
       if UtiliseValeurs
       then
           sOOoFieldContent:= Field.DisplayText
       else
           sOOoFieldContent:= sOOoFieldName;

       Traite_Field( sOOoFieldName, sOOoFieldContent, CreeTextFields);
       end;

     TextFields.refresh;
     Result:= True;
end;

function TOOoTextFieldsCreator.Execute_interne( _Document: Variant;
                                                FieldName,
                                                FieldContent: String;
                                                CreeTextFields: Boolean): Boolean;
begin
     Init( _Document);

     Traite_Field( FieldName, FieldContent, CreeTextFields);

     TextFields.refresh;
     Result:= True;
end;

function TOOoTextFieldsCreator.Execute_Modele ( _Document: Variant;
                                                D: TDataset;
                                                Nouveau_Modele: Boolean): Boolean;
begin
     Result:= Execute_interne( _Document, D, False, Nouveau_Modele);
end;

function TOOoTextFieldsCreator.Execute        ( _Document: Variant;
                                                D: TDataset): Boolean;
begin
     Result:= Execute_interne( _Document, D, True, False);
end;

function TOOoTextFieldsCreator.Execute_Modele ( _Document: Variant;
                                                FieldName: String;
                                                Nouveau_Modele: Boolean): Boolean;
begin
     Result:= Execute_interne( _Document, FieldName, FieldName, Nouveau_Modele);
end;

function TOOoTextFieldsCreator.Execute        ( _Document: Variant;
                                                FieldName,
                                                FieldContent: String): Boolean;
begin
     Result:= Execute_interne( _Document, FieldName, FieldContent, False);
end;

function TOOoTextFieldsCreator.Existe( _Document: Variant; FieldName: String): Boolean;
var
   FieldPathName: String;
begin
     Init( _Document);
     FieldPathName:= 'com.sun.star.text.FieldMaster.User.'+FieldName;
     Result:= TFM.hasByName( FieldPathName)
end;

initialization
              OOoTextFieldsCreator:= TOOoTextFieldsCreator.Create;
finalization
              FreeAndNil( OOoTextFieldsCreator);
end.
