unit uOOoTextTableManager;
{                                                                             |
    Part of package pOpenOfficeReport                                         |
                                                                              |
    Author: Jean SUZINEAU <Jean.Suzineau@wanadoo.fr>                          |
            partly as freelance: http://www.mars42.com                        |
        and partly as employee : http://www.batpro.com                        |
    Contact: gilles.doutre@batpro.com                                         |
                                                                              |
    Copyright (C) 2004  Jean SUZINEAU - MARS42                                |
    Copyright (C) 2004  Cabinet Gilles DOUTRE - BATPRO                        |
                                                                              |
    See pOpenOfficeReport.dpk for full copyright notice.                      |
|                                                                             }

interface

uses
    SysUtils, Classes, DB, Grids, Variants, Dialogs, Windows,
    uUNO_ControlCharacter,
    uUNO_DeskTop,
    uUNO_FieldMaster_User,
    uUNO_TextField_User,
    uUNO_TextTable,
    uOOoSurTitre,
    uOOoMerge;

type
 TOOoTextTableManager
 =
  class
  private
    Document,
    Text,
    Cursor,
    StyleFamilies,
    ParagraphStyles: Variant;
    procedure Init( _Document: Variant);
    function  NomStyle_from_Field         ( D: TDataset; I: Integer): String;
    function  NomStyleColonne_from_Dataset( D: TDataset            ): String;
  // Cration d'une table  partir d'un dataset
  private
    function OOoTextTableName_from_Dataset( D: TDataset): String;
  public
    function Execute_Modele( _Document: Variant; D: TDataset;
                             Nouveau_Modele: Boolean): Boolean; overload;
    function Remplit( _Document: Variant; D: TDataset;
                      OOoSurTitre: TOOoSurTitre = nil;
                      OOoMerge   : TOOoMerge    = nil): Boolean; overload;
  // Cration d'une table  partir d'un StringGrid
  private
    function OOoTextTableName_from_StringGrid( sg: TStringGrid): String;
    function  NomStyleColonne_from_StringGrid( sg: TStringGrid): String;
  public
    function Nouveau( _Document: Variant; sg: TStringGrid): Boolean; overload;
    function Execute_Modele( _Document: Variant; sg: TStringGrid;
                             Nouveau_Modele: Boolean): Boolean; overload;
    function Remplit( _Document: Variant; sg: TStringGrid;
                      OOoSurTitre: TOOoSurTitre = nil): Boolean; overload;
  end;

var
   OOoTextTableManager: TOOoTextTableManager= nil;

function CellName_from_XY( X, Y: Integer): String;

implementation

uses
    uUNO_ParagraphStyle(*,
    OOoXRay*);

function CellName_from_XY( X, Y: Integer): String;
begin
     Result:=  Chr(Ord('A')+X) + IntToStr(Y+1);
end;

function RangeName_from_Rect( Left, Top, Right, Bottom: Integer): String;
begin
     Result:=  CellName_from_XY( Left , Top   )
              +':'
              +CellName_from_XY( Right, Bottom);
end;

{ TOOoTextTableManager }

procedure TOOoTextTableManager.Init( _Document: Variant);
begin
     Document:= _Document;
     Text:= Document.Text;
     Cursor:= Text.createTextCursor;
     StyleFamilies:= Document.StyleFamilies;
     ParagraphStyles:= StyleFamilies.getByName('ParagraphStyles');
end;

function TOOoTextTableManager.OOoTextTableName_from_Dataset( D: TDataset): String;
var
   D_Owner: TComponent;
begin
     Result:= D.Name;
     D_Owner:= D.Owner;
     if Assigned( D_Owner)
     then
         Result:= D_Owner.Name+'_'+Result;
end;

function TOOoTextTableManager.NomStyle_from_Field( D: TDataset; I: Integer): String;
begin
     Result:= '_'+D.Name+'_'+D.Fields[I].FieldName;
end;

function TOOoTextTableManager.NomStyleColonne_from_Dataset( D: TDataset): String;
begin
     Result:= '_'+D.Name+'_Style_Colonne';
end;

function TOOoTextTableManager.Execute_Modele( _Document: Variant; D: TDataset;
                                                Nouveau_Modele: Boolean): Boolean;
var
   NomTable: String;
   NomStyleColonne: String;
   NomStyle: String;
   tt: TUNO_TextTable;
   UNO_ParagraphStyle: TUNO_ParagraphStyle;
   I: Integer;
begin
     Init( _Document);

     NomTable:= OOoTextTableName_from_Dataset( D);

     NomStyleColonne:= NomStyleColonne_from_Dataset( D);
     if not ParagraphStyles.hasByName( NomStyleColonne)
     then
         begin
         UNO_ParagraphStyle:= TUNO_ParagraphStyle.Create( Document, NomStyleColonne);
         FreeAndNil( UNO_ParagraphStyle);
         end;

     for I:= 0 to D.FieldCount - 1
     do
       begin
       NomStyle:= NomStyle_from_Field( D, I);
       if not ParagraphStyles.hasByName( NomStyle)
       then
           begin
           UNO_ParagraphStyle:= TUNO_ParagraphStyle.Create( Document, NomStyle);
           UNO_ParagraphStyle.ole.setParentStyle( NomStyleColonne);
           FreeAndNil( UNO_ParagraphStyle);
           end;
       end;

     if Nouveau_Modele
     then
         begin
         Cursor.gotoEnd( False);
         Text.insertControlCharacter( Cursor,
                                      UNO_ControlCharacter_PARAGRAPH_BREAK,
                                      False);
         Text.getEnd.SetString( NomTable+': ');
         Cursor.gotoEnd( False);
         Text.insertControlCharacter( Cursor,
                                      UNO_ControlCharacter_PARAGRAPH_BREAK,
                                      False);
         tt:= TUNO_TextTable.Create( Document);
         try
            tt.Name:= NomTable;
            tt.initialize( 2, 1);
            Text.insertTextContent( Cursor, tt.ole, False);
         finally
                FreeAndNil( tt);
                end;
         end;

     Result:= True;
end;

function TOOoTextTableManager.Remplit( _Document: Variant;
                                       D: TDataset;
                                       OOoSurTitre: TOOoSurTitre = nil;
                                       OOoMerge   : TOOoMerge    = nil): Boolean;
var
   NomTable: String;
   NbColonnes: Integer;
   I, iColonne, J, JTitre: Integer;
   F: TField;
   NomStyle, sCellValue: String;
   ColumnLengths: array of Integer;
   ColumnEnds: array of Integer;
   ColumnLengths_Sum, ColumnWidths_Sum, LastEnd: Integer;

   //Merge
   IMerge: Integer;
   Merge_OK: Boolean;

   //Surtitre
   OffSet_SurTitre: Integer;
   Debut, Fin: Integer;
   CellName: String;

   Table,
   Rows,
   Columns,
   CellCursor,
   TextCursor,
   TableColumnSeparators,
   Anchor: Variant;
begin
     Init( _Document);

     NbColonnes:= 0;
     for I:= 0 to D.FieldCount - 1
     do
       if D.Fields[I].Visible
       then
           Inc( NbColonnes);

     NomTable:= OOoTextTableName_from_Dataset( D);
     Table:= Document.getTextTables.getByName( NomTable);
     Columns:= Table.getColumns;
     Rows   := Table.getRows;

     SetLength( ColumnLengths, NbColonnes);
     SetLength( ColumnEnds   , NbColonnes);

     //Titres
     if NbColonnes > 1
     then
         Columns.insertByIndex(J,NbColonnes-1);

     J:= 0;
     //Ajout ventuel ligne pour SurTitres
     if Assigned( OOoSurTitre)
     then
         begin
         Rows.insertByIndex(J,1);
         Inc( J);
         OffSet_SurTitre:= 1;
         end
     else
         OffSet_SurTitre:= 0;

     JTitre:= J;
     iColonne:= 0;
     for I:= 0 to D.FieldCount - 1
     do
       if D.Fields[I].Visible
       then
           begin
           F:= D.Fields[I];
           sCellValue:= F.DisplayLabel;
           ColumnLengths[iColonne]:= F.DisplayWidth;

           CellCursor:= Table.getCellByPosition( iColonne, J);
           TextCursor:= CellCursor.CreateTextCursor;
           CellCursor.insertString( TextCursor, sCellValue, false);
           Inc( iColonne);
           end;
     //Lignes
     D.First;
     while not D.Eof
     do
       begin
       Inc(J);
       if J > OffSet_SurTitre + 1
       then
           Rows.insertByIndex(J,1);
       iColonne:= 0;
       for I:= 0 to D.FieldCount - 1
       do
         if D.Fields[I].Visible
         then
             begin
             sCellValue:= D.Fields[I].DisplayText;
             NomStyle  := NomStyle_from_Field( D, I);

             //CellCursor:= Table.getCellByName(sCellName);
             CellCursor:= Table.getCellByPosition( iColonne, J);
             TextCursor:= CellCursor.CreateTextCursor;
             if ParagraphStyles.hasByName( NomStyle)
             then
                 TextCursor.ParaStyleName:= NomStyle;
             CellCursor.insertString( TextCursor, sCellValue, false);
             Inc( iColonne);
             end;
       D.Next;
       end;

     //Gestion de la largeur des colonnes
     ColumnLengths_Sum:= 0;
     for I:= 0 to NbColonnes -1
     do
       Inc( ColumnLengths_Sum, ColumnLengths[I]);
     ColumnWidths_Sum:= Table.TableColumnRelativeSum;
     LastEnd:= 0;
     for I:= 0 to NbColonnes -1
     do
       begin
       LastEnd:= LastEnd+MulDiv( ColumnLengths[I], ColumnWidths_Sum, ColumnLengths_Sum);
       ColumnEnds[I]:= LastEnd;
       end;
     TableColumnSeparators:= Table.TableColumnSeparators;
     for I:= VarArrayLowBound (TableColumnSeparators,1)
         to  VarArrayHighBound(TableColumnSeparators,1)
     do
       TableColumnSeparators[I].Position:= ColumnEnds[I];
     Table.TableColumnSeparators:= TableColumnSeparators;

     if Assigned( OOoMerge)
     then
         while J > JTitre
         do
           begin
           for I:= High(OOoMerge.Debut) downto Low(OOoMerge.Debut)
           do
             begin
             Debut     := OOoMerge.Debut  [I];
             Fin       := OOoMerge.Fin    [I];

             Merge_OK:= True;
             for IMerge:= Debut+1 to Fin
             do
               begin
               CellName:= CellName_from_XY(IMerge, J);
               CellCursor:= Table.getCellByName( CellName);
               sCellValue:= CellCursor.getString;
               Merge_OK:= sCellValue = '';
               if not Merge_OK then break;
               end;

             if Merge_OK
             then
                 begin
                 //CellName:= CellName_from_XY(Debut, J);
                 //CellCursor:= Table.getCellByName( CellName);
                 //sCellValue:= CellCursor.getFormula;

                 CellName:= CellName_from_XY(Debut, J);
                 CellCursor:= Table.createCursorByCellName( CellName);
                 CellCursor.gotoCellByName( CellName_from_XY(Fin, J), true);
                 //xray( CellCursor);
                 CellCursor.mergeRange;

                 //CellCursor:= Table.getCellByName( CellName_from_XY(Debut, J));
                 //TextCursor:= CellCursor.CreateTextCursor;
                 //CellCursor.insertString( TextCursor, sCellValue, false);
                 end;
             end;
           Dec( J);
           end;
     //SurTitres
     if Assigned( OOoSurTitre)
     then
         begin
         J:= 0;
         for I:= High(OOoSurTitre.Libelle) downto Low(OOoSurTitre.Libelle)
         do
           begin
           sCellValue:= OOoSurTitre.Libelle[I];
           Debut     := OOoSurTitre.Debut  [I];
           Fin       := OOoSurTitre.Fin    [I];

           CellName:= CellName_from_XY(Debut, J);
           CellCursor:= Table.createCursorByCellName( CellName);
           CellCursor.gotoCellByName( CellName_from_XY(Fin, J), true);
           //xray( CellCursor);
           CellCursor.mergeRange;

           CellCursor:= Table.getCellByName( CellName_from_XY(Debut, J));
           TextCursor:= CellCursor.CreateTextCursor;
           CellCursor.insertString( TextCursor, sCellValue, false);
           end;
         end;

     Result:= True;
end;

function TOOoTextTableManager.OOoTextTableName_from_StringGrid( sg: TStringGrid): String;
var
   sg_Owner: TComponent;
begin
     Result:= sg.Name;
     sg_Owner:= sg.Owner;
     if Assigned( sg_Owner)
     then
         Result:= sg_Owner.Name+'_'+Result;
end;

function TOOoTextTableManager.NomStyleColonne_from_StringGrid( sg: TStringGrid): String;
begin
     Result:= OOoTextTableName_from_StringGrid( sg)+'_Style_Colonne';
end;

function TOOoTextTableManager.Execute_Modele( _Document: Variant; sg: TStringGrid;
                                                Nouveau_Modele: Boolean): Boolean;
var
   NomTable: String;
   NomStyle: String;
   NomStyleColonne: String;
   tt: TUNO_TextTable;
   UNO_ParagraphStyle: TUNO_ParagraphStyle;
   I: Integer;
begin
     Init( _Document);

     NomTable:= OOoTextTableName_from_StringGrid( sg);

     NomStyleColonne:= NomStyleColonne_from_StringGrid( sg);
     if not ParagraphStyles.hasByName( NomStyleColonne)
     then
         begin
         UNO_ParagraphStyle:= TUNO_ParagraphStyle.Create( Document, NomStyleColonne);
         FreeAndNil( UNO_ParagraphStyle);
         end;

     for I:= 0 to sg.ColCount-1
     do
       begin
       NomStyle:= NomTable+'_Colonne_'+IntToStr( I);
       if not ParagraphStyles.hasByName( NomStyle)
       then
           begin
           UNO_ParagraphStyle:= TUNO_ParagraphStyle.Create( Document, NomStyle);
           UNO_ParagraphStyle.ole.setParentStyle( NomStyleColonne);
           FreeAndNil( UNO_ParagraphStyle);
           end;
       end;

     if Nouveau_Modele
     then
         begin
         Cursor.gotoEnd( False);
         Text.insertControlCharacter( Cursor,
                                      UNO_ControlCharacter_PARAGRAPH_BREAK,
                                      False);
         Text.getEnd.SetString( NomTable+': ');
         Cursor.gotoEnd( False);
         Text.insertControlCharacter( Cursor,
                                      UNO_ControlCharacter_PARAGRAPH_BREAK,
                                      False);
         tt:= TUNO_TextTable.Create( Document);
         try
            tt.Name:= NomTable;
            tt.initialize( 2, 1);
            Text.insertTextContent( Cursor, tt.ole, False);
         finally
                FreeAndNil( tt);
                end;
         end;

     Result:= True;
end;

function TOOoTextTableManager.Nouveau( _Document: Variant; sg: TStringGrid): Boolean;
var
   NomTable: String;
   NomStyle: String;
   tt: TUNO_TextTable;
   I: Integer;
   UNO_ParagraphStyle: TUNO_ParagraphStyle;
begin
     Init( _Document);

     NomTable:= OOoTextTableName_from_StringGrid( sg);

     for I:= 0 to sg.ColCount-1
     do
       begin
       NomStyle:= NomTable+'_Colonne_'+IntToStr( I);
       UNO_ParagraphStyle:= TUNO_ParagraphStyle.Create( Document, NomStyle);
       FreeAndNil( UNO_ParagraphStyle);
       end;

     Cursor.gotoEnd( False);
     Text.insertControlCharacter( Cursor,
                                  UNO_ControlCharacter_PARAGRAPH_BREAK,
                                  False);
     Text.getEnd.SetString( NomTable+': ');
     Cursor.gotoEnd( False);
     Text.insertControlCharacter( Cursor,
                                  UNO_ControlCharacter_PARAGRAPH_BREAK,
                                  False);
     tt:= TUNO_TextTable.Create( Document);
     try
        tt.Name:= NomTable;
        tt.initialize( 2, 1);
        Text.insertTextContent( Cursor, tt.ole, False);
     finally
            FreeAndNil( tt);
            end;

     Result:= True;
end;

function TOOoTextTableManager.Remplit( _Document: Variant; sg: TStringGrid;
                                       OOoSurTitre: TOOoSurTitre = nil): Boolean;
var
   NomTable: String;
   NbColonnes, NbLignes: Integer;
   I, J: Integer;
   sCellValue: String;
   NomStyle: String;
   ColumnLengths: array of Integer;
   ColumnEnds: array of Integer;
   ColumnLength, ColumnLengths_Sum, ColumnWidths_Sum, LastEnd: Integer;

   OffSet_SurTitre: Integer;
   Debut, Fin: Integer;
   CellName: String;

   Table,
   Rows,
   Columns,
   CellCursor,
   TextCursor,
   StyleFamilies,
   ParagraphStyles,
   TableColumnSeparators: Variant;

begin
     Init( _Document);

     NbColonnes:= sg.ColCount;
     NbLignes  := sg.RowCount;

     NomTable:= OOoTextTableName_from_StringGrid( sg);
     Table   := Document.getTextTables.getByName( NomTable);
     Columns := Table.getColumns;
     Rows    := Table.getRows;
     StyleFamilies:= Document.StyleFamilies;
     ParagraphStyles:= StyleFamilies.getByName('ParagraphStyles');

     SetLength( ColumnLengths, NbColonnes);
     SetLength( ColumnEnds   , NbColonnes);
     if NbColonnes > 1
     then
         Columns.insertByIndex(1,NbColonnes-1);

     if NbLignes > 2
     then
         Rows   .insertByIndex(1,NbLignes  -2);

     //Ajout ventuel ligne pour SurTitres
     if Assigned( OOoSurTitre)
     then
         begin
         Rows.insertByIndex(0,1);
         OffSet_SurTitre:= 1;
         end
     else
         OffSet_SurTitre:= 0;

     for J:= 0 to sg.RowCount - 1
     do
       begin
       for I:= 0 to NbColonnes - 1
       do
         begin
         sCellValue:= sg.Cells[ I, J];
         ColumnLength:= Length( sCellValue);
         if ColumnLengths[I] < ColumnLength
         then
             ColumnLengths[I]:= ColumnLength;

         CellCursor:= Table.getCellByPosition( I, OffSet_SurTitre+J);
         TextCursor:= CellCursor.CreateTextCursor;
         if J > 0
         then
             begin
             NomStyle:= NomTable+'_Colonne_'+IntToStr( I);
             if ParagraphStyles.hasByName( NomStyle)
             then
                 TextCursor.ParaStyleName:= NomStyle;
             end;
         CellCursor.insertString( TextCursor, sCellValue, false);
         end;
       end;

     ColumnLengths_Sum:= 0;
     for I:= 0 to NbColonnes -1
     do
       Inc( ColumnLengths_Sum, ColumnLengths[I]);
     ColumnWidths_Sum:= Table.TableColumnRelativeSum;
     LastEnd:= 0;
     for I:= 0 to NbColonnes -1
     do
       begin
       LastEnd:= LastEnd+MulDiv( ColumnLengths[I], ColumnWidths_Sum, ColumnLengths_Sum);
       ColumnEnds[I]:= LastEnd;
       end;
     TableColumnSeparators:= Table.TableColumnSeparators;
     for I:= VarArrayLowBound (TableColumnSeparators,1)
         to  VarArrayHighBound(TableColumnSeparators,1)
     do
       TableColumnSeparators[I].Position:= ColumnEnds[I];
     Table.TableColumnSeparators:= TableColumnSeparators;

     //SurTitres
     if Assigned( OOoSurTitre)
     then
         begin
         J:= 0;
         for I:= High(OOoSurTitre.Libelle) downto Low(OOoSurTitre.Libelle)
         do
           begin
           sCellValue:= OOoSurTitre.Libelle[I];
           Debut     := OOoSurTitre.Debut  [I];
           Fin       := OOoSurTitre.Fin    [I];

           CellName:= CellName_from_XY(Debut, J);
           CellCursor:= Table.createCursorByCellName( CellName);
           CellCursor.gotoCellByName( CellName_from_XY(Fin, J), true);
           //xray( CellCursor);
           CellCursor.mergeRange;

           CellCursor:= Table.getCellByName( CellName_from_XY(Debut, J));
           TextCursor:= CellCursor.CreateTextCursor;
           CellCursor.insertString( TextCursor, sCellValue, false);
           end;
         end;

     Result:= True;
end;

initialization
              OOoTextTableManager:= TOOoTextTableManager.Create;
finalization
              FreeAndNil( OOoTextTableManager);
end.
